import os
from io import BytesIO
from tempfile import TemporaryDirectory
from zipfile import ZipFile

from translator_template.app import generate, generate_to_file
from translator_template.templates import EPILOG, PROLOG

TEST_DIR = os.path.dirname(os.path.abspath(__file__))


def test_generate_to_file():
    with TemporaryDirectory() as dir:
        in_file = os.path.join(TEST_DIR, "data", "test.txt")
        out_file = os.path.join(dir, "test.docx")
        generate_to_file(in_file, out_file)


def test_generate():
    in_file = os.path.join(TEST_DIR, "data", "test.txt")
    zip_bytes = generate(in_file)
    zip_file = BytesIO(zip_bytes)
    with ZipFile(zip_file) as zip:
        with zip.open("word/document.xml") as docx:
            xml = docx.read().decode("utf8")
            assert xml.startswith(PROLOG)
            assert xml.endswith(EPILOG)



import argparse
from io import BytesIO
from os import path
from typing import IO, Union
from zipfile import ZipFile

from .parse import read
from .templates import EPILOG, PROLOG, PAGE, PANEL, QUOTE


def generate_to_file(in_file: str, out_file: Union[str, IO]) -> None:
    """Read the input specification from the in_file file, and generate a Word document at out_file."""
    bytes = generate(in_file)
    with open(out_file, "wb") as f:
        f.write(bytes)


def generate(in_file: str) -> bytes:
    """Read the input specification from the in_file file, and generate a Word document to a byte buffer."""
    with _load_zip() as stream:
        with ZipFile(stream, mode="a") as zip:
            doc = _generate(in_file)
            zip.writestr("word/document.xml", doc)
        return stream.getvalue()


def _load_zip() -> IO:
    zip_path = path.join(path.dirname(path.abspath(__file__)), "data", "test.docx")
    with open(zip_path, "rb") as f:
        stream = BytesIO(f.read())
    return stream


def _generate(in_file: str) -> str:
    chunks = [PROLOG]
    with open(in_file, "rt") as f:
        for page, panels in read(f):
            chunks.append(PAGE.format(page))
            for panel in range(panels):
                chunks.append(PANEL.format(panel + 1))
                chunks.append(QUOTE)
    chunks.append(EPILOG)
    return "".join(chunks)


def main():
    parser = argparse.ArgumentParser(
        description="Generate a Word Document from page/panel specification"
    )
    parser.add_argument("--in-file", help="input file name", required=True)
    parser.add_argument("--out-file", help="output file name")
    args = parser.parse_args()
    if args.out_file:
        out_file = args.out_file
    else:
        root, _ = path.splitext(args.in_file)
        out_file = root + ".docx"
    generate_to_file(args.in_file, out_file)



PROLOG = (
    "<?xml version='1.0' encoding='UTF-8' standalone='yes'?>"
    '<w:document xmlns:wpc="http://schemas.microsoft.com/office/word/2010/wordprocessingCanvas" xmlns:mo="http://schemas.microsoft.com/office/mac/office/2008/main" xmlns:mc="http://schemas.openxmlformats.org/markup-compatibility/2006" xmlns:mv="urn:schemas-microsoft-com:mac:vml" xmlns:o="urn:schemas-microsoft-com:office:office" xmlns:r="http://schemas.openxmlformats.org/officeDocument/2006/relationships" xmlns:m="http://schemas.openxmlformats.org/officeDocument/2006/math" xmlns:v="urn:schemas-microsoft-com:vml" xmlns:wp14="http://schemas.microsoft.com/office/word/2010/wordprocessingDrawing" xmlns:wp="http://schemas.openxmlformats.org/drawingml/2006/wordprocessingDrawing" xmlns:w10="urn:schemas-microsoft-com:office:word" xmlns:w="http://schemas.openxmlformats.org/wordprocessingml/2006/main" xmlns:w14="http://schemas.microsoft.com/office/word/2010/wordml" xmlns:wpg="http://schemas.microsoft.com/office/word/2010/wordprocessingGroup" xmlns:wpi="http://schemas.microsoft.com/office/word/2010/wordprocessingInk" xmlns:wne="http://schemas.microsoft.com/office/word/2006/wordml" xmlns:wps="http://schemas.microsoft.com/office/word/2010/wordprocessingShape" mc:Ignorable="w14 wp14">'
    "<w:body>"
)

EPILOG = (
    '<w:sectPr w:rsidR="00FC693F" w:rsidRPr="0006063C" w:rsidSect="00034616">'
    '<w:pgSz w:w="12240" w:h="15840"/>'
    '<w:pgMar w:top="1440" w:right="1800" w:bottom="1440" w:left="1800" w:header="720" w:footer="720" w:gutter="0"/>'
    '<w:cols w:space="720"/>'
    '<w:docGrid w:linePitch="360"/>'
    "</w:sectPr>"
    "</w:body>"
    "</w:document>"
)

PAGE = (
    "<w:p>"
    "<w:pPr>"
    '<w:pStyle w:val="PagePara"/>'
    "</w:pPr>"
    "<w:r>"
    "<w:t>{}</w:t>"
    "</w:r>"
    "</w:p>"
)

PANEL = (
    "<w:p>"
    "<w:pPr>"
    '<w:pStyle w:val="PanelPara"/>'
    "</w:pPr>"
    "<w:r>"
    "<w:t>PANEL {}</w:t>"
    "</w:r>"
    "</w:p>"
)

QUOTE = '<w:p><w:pPr><w:pStyle w:val="QuotePara"/></w:pPr></w:p>'
